(use-modules ((srfi srfi-1)  :select ((for-each . for-each-1) circular-list)))
(use-modules ((srfi srfi-13) :select (string-append/shared)))
(use-modules ((ice-9 q)      :select (make-q enq!)))
(use-modules ((ice-9 rdelim) :select (read-line)))

(define (write-file port-or-path text)
  "Write the given text to the given port or path, and close the port."
  (let ((port (if (port? port-or-path)
                port-or-path
                (open port-or-path (logior O_WRONLY O_CREAT O_EXCL)))))
    (display text port)
    (close-port port)))

(define (slurp port-or-path)
  "Read and return all text from the given port or path, and close the port."
  (let ((port (if (port? port-or-path)
                port-or-path
                (open port-or-path O_RDONLY))))
    (let loop ((buffer ""))
      (if (eof-object? (peek-char port))
        (begin
          (close-port port)
          buffer)
        (loop (string-append/shared buffer (drain-input port)))))))

(define (lines-until-empty port-or-path)
  "\
Return a list of lines read from the given port or path, up to the first
empty line."
  (let ((port (if (port? port-or-path)
                port-or-path
                (open port-or-path O_RDONLY)))
        (lines (make-q)))
    (let loop ()
      (let ((line (read-line port)))
        (if (or (eof-object? line) (string-null? line))
          (begin
            (close-port port)
            (car lines))
          (begin
            (enq! lines line)
            (loop)))))))

(define (cat ports-or-paths . rest)
  "\
Write the contents of each port or path to the given output port, or the
current output port by default, and closes the input ports."
  (let ((output-port (if (null? rest) (current-output-port) (car rest))))
    (for-each-1 cat-port-or-path ports-or-paths (circular-list output-port))))
(define (cat-port-or-path port-or-path output-port)
  (cat-port (if (port? port-or-path)
              port-or-path
              (open port-or-path O_RDONLY))
            output-port))
(define (cat-port input-port output-port)
  (while (not (eof-object? (peek-char input-port)))
    (display (drain-input input-port) output-port))
  (close-port input-port))
